//***********************************************************************
//* FILE:        clock.e
//* BY:          Jerel Canales
//* COMPANY:     Texas Instruments
//* PURPOSE:     Bucket for clock events
//*              
//*              
//*
//* 04/26/01     jerelc   Initial code
//***********************************************************************
/*
// CONTENTS (you can search for these words to jump to the appropriate section):
//     OUTPUTS
//     EXAMPLE

// OUTPUTS:
// event fall;      // falling edge of gated clock (gated on reset)
// event rise;      // rising edge of gated clock (gated on reset)
// event raw_fall;  // falling edge of raw clock 
// event raw_rise;  // rising edge of raw clock
   
// EXAMPLE:
//
//    unit TESTBENCH {
//       clk      : clock_stub;
//       event rclk is rise('clkin') @sim;
//       event rclk is fall('clkin') @sim;
//
//       on rclk { 
//         if ('reset'==1'b1) then { 
//            emit clk.rise;
//         };
//         emit clk.raw_rise;
//       };
//
//       on fclk { 
//         if ('reset'==1'b1) then {
//            emit clk.fall;
//         };
//         emit clk.raw_fall;
//       };
//    };//end TESTBENCH
      
//    struct proto_chk_s {
//       clk : clock_stub;
//       event rclk is cycle @clk.rise;
//       event fclk is cycle @clk.fall;
//        .         .
//        .         .
//        .         .
//    };

//    extend TESTBENCH {
//       xyz_pc   : proto_chk_s;
//       keep xyz_pc.clk == clk;
//    };
//
   
<'
   
///////////////////////////////////////////////////////////////////////////////////
//
//  ####   #        ####    ####   #    #           ####    #####  #    #  #####
// #    #  #       #    #  #    #  #   #           #          #    #    #  #    #
// #       #       #    #  #       ####             ####      #    #    #  #####
// #       #       #    #  #       #  #                 #     #    #    #  #    #
// #    #  #       #    #  #    #  #   #           #    #     #    #    #  #    #
//  ####   ######   ####    ####   #    # #######   ####      #     ####   #####
//
// Clocking events
   
struct clock_stub {

   event fall;        // falling edge of gated clock
   event rise;        // rising edge of gated clock
   event raw_fall;    // falling edge of raw clock
   event raw_rise;    // rising edge of raw clock
};

'>
   
/////////////////////////////////////////////////////////////////////////
//
//  #######
//     #     #    #  ######          ######  #    #  #####
//     #     #    #  #               #       ##   #  #    #
//     #     ######  #####           #####   # #  #  #    #
//     #     #    #  #               #       #  # #  #    #
//     #     #    #  #               #       #   ##  #    #
//     #     #    #  ######          ######  #    #  #####
//
/////////////////////////////////////////////////////////////////////////
   
