//********************************************************************************
//
//  Filename: cpubus_definitions.e
//
//  Description: definitions
//
//  Major Revision Descriptions:
//  oishi          09/08/02         copied from UMA1.0 branch
//  oishi          09/09/02         removed bottom signals
//                                  removed mdp related functions
//                                  removed ack related functions
//                                  removed data dismiss related functions
//                                  removed stall signals
//  oishi          09/11/02         changed bdbus width to 32bits
//                                  added following qualifiers;
//                                    cpu_dblcoef_tr/cpu_rdbyte_tr/cpu_rdlsbe_tr
//  oishi          09/12/02         added MemoryProtection related signals
//                                  added cpu_avis_tr siganl
//  oishi          09/13/02         modified resetn to cpu_resetn and gl_resetn
//                                  following signals are added;
//                                    gl_buserror_nr/gl_debugerror_nr
//                                    gl_nmi_nr/gl_int_nr/cpu_idlewakeup_ta
//  oishi          09/27/02         added configuration output
//                                  added MMR_IMR1,MMR_IFR1 and iack signal
//
//  oishi          09/30/02         added gl_idlecpu_tr/gl_idlecpud_tr signal
//  oishi          10/01/02         removed LockAccess_SingleWrite/LockAccess_SingleRead
//  oishi          10/04/02         supports cpu_suspend_nr signal
//                                  added checking function of signal connection.
//  oishi          11/22/02         updated dbus_access_type and ebus_access_type
//                                  for supporting IceMaker XDMA access
//  oishi          11/25/02         added interrupt_enable switch for configuration
//  oishi          12/13/02         added perrack signal
//  hiraoka        03/25/03         add following signal
//                                    gl_homnsam_tr(3 DOWNTO 0)
//                                    cpu_homnsam_tr(3 DOWNTO 0)
//                                    gl_hompnsamp_init_tr
//                                    gl_homrnsamr_init_tr
//                                    gl_homxnsamx_init_tr
//                                    gl_homynsamy_init_tr
//  hiraoka        07/09/03         add module_category_t for speed improvement on get_current_value()
//*********************************************************************************
<'
----------------------------------------
-- MMR address definitions(0x00~0x5f) --
----------------------------------------
define MMR_IMR0         0x0000;
define MMR_IFR0         0x0001;
define MMR_IMR1         0x0045;
define MMR_IFR1         0x0046;
define MMR_ST0_55       0x0002;
define MMR_ST1_55       0x0003;
define MMR_ST3_55       0x0004;

define SHADOW_HOMSAM    0x0;

-----------------------------------
-- IceMaker MMR area in io space --
-----------------------------------
-- Note that icemake module is placed in the laijin.
-- So ready signal for request of icemaker's mmr is returned 
-- icemaker module that is in the laijin.
-- However these request is happen to outside of the laijin.
-- Then outside module must not return the ready siganl
-- for these requests. (only laijin2.x)

-- this is MGS3 definitions --
--define ICEMAKER_IOMMR   [ 0x0420..0x0427,
--                          0x0440..0x0447,
--                          0x05fc..0x05ff ];

-- accoding to LaijinEmulationMegacell(UMA1.0) Design Note --
define ICEMAKER_IOMMR   [ 0x0400..0x05ff ];

---------------------------------------------------
-- cpu access instructions list for this CPU BFM --
---------------------------------------------------
                                                          --   B  C  D  E  F     qualifiers
type data_access_instruction : [ None,                    --   .  .  .  .  .     
                                 SingleRead,              --   .  .  o  .  .     iodreq, wrbyte, wrlsbe
                                 DualRead,                --   .  o  o  .  .     iodreq, wrbyte, wrlsbe
                                 DoubleRead,              --   .  o  o  .  .     doubler
                                 SingleWrite,             --   .  .  .  o  .
                                 DualWrite,               --   .  .  .  o  o
                                 DoubleWrite,             --   .  .  .  o  o     doublew
                                 LockAccess,              --   .  .  o  o  .     lock(D/Ebus is locked)
                                 CoefRead                 --   o  .  .  .  .
                               ];

type access_space     : [ io, memory ];
type access_width     : [ high_byte, low_byte, word ];
type access_double    : [ double, word ];
type access_lock      : [ disable, enable ];
type activate_bus     : [ active, no_active, finished ];

----------------------
-- bus access types --
----------------------
-- 
-- for emulation access, following types are exists;
--   
--   IceMakerMU/DT-DMA access : pbus_access_type = debug
--   (supports in 2.x/3.x)      dbus_access_type = debug
--                              ebus_access_type = debug_prog_write or debug_data_write
--   
--   IceMaker XDMA access     : dbus_access_type = xdma
--   (supports in 3.x)          ebus_access_type = xdma
--   
type pbus_access_type : [ no_debug, debug ];
type dbus_access_type : [ no_debug, debug, xdma ];
type ebus_access_type : [ no_debug, debug_prog_write, debug_data_write, xdma ];

--  ----------------------------------------------------------------------------------------------------
--  Instructions                                      B  C  D  E  F     qualifiers
--  --------------------------------------------------------------------------------------------------
--  None                                              -  -  -  -  -     
--  
--  SingleRead(mtype,width,dabus)                     -  -  o  -  -     iodreq, rdbyte, rdlsbe
--  DualRead(mtype,width,cabus,dabus)                 -  o  o  -  -     iodreq, rdbyte, rdlsbe
--  DoubleRead(mtype,dabus)                           -  o  o  -  -     doubler
--  
--  SingleWrite(mtype,width,eabus,edbus)              -  -  -  o  -     ioereq, wrbyte, wrlsbe
--  DualWrite(mtype,eabus,fabus,edbus,fdbus)          -  -  -  o  o     ioereq, wrbyte, wrlsbe
--  DoubleWrite(mtype,eabus,edbus,fdbus)              -  -  -  o  o     doublew
--  
--  LockAccess(mtype,deabus)                          -  -  o  o  -     lock(D/Ebus is locked),iodreq,ioereq
--  
--  CoefRead(width, babus, dabus)                     o  -  -  -  -     dblcoef(word/double)
--  --------------------------------------------------------------------------------------------------
--
--  [mtype = io or memory](for Ebus/Dbus)
--  [width = high_byte, low_byte, word](for Dbus/Ebus)
--  
--  InstCoefGroup  [ Nop or CoefRead    ]
--  InstReadGroup  [ Nop or SingleRead  or DualRead  or DoubleRead  ]
--  InstWriteGroup [ Nop or SingleWrite or DualWrite or DoubleWrite ]
--  InstLockGroup  [ Nop or LockAccess or LockAccessSingleWrite or LockAccessSingleRead ]
--  
--  lock : access_lock
--  if lock == disable then [InstCoefGroup + InstReadGroup + InstWriteGroup]
--  if lock == enable  then [InstCoefGroup + InstLockGroup]

type program_access_instruction : [ None, ProgramRead ];

type module_category_t : [ NONE, MASTER, SLAVE ];
--------------------------------------------
-- method : defined cpubus configurations --
--------------------------------------------
unit CPUBUS_CONFIGURATIONS {
    
    core_version  : string;
    id            : string;
    
    driver_delay  : uint;
    pbus_start_address :  uint(bits : 24);
    
    -- logger on/off --
    logger_on    : bool;
    keep soft logger_on == TRUE;
    
    -- unknown driving --
    unknown_drive : bool;
    keep soft unknown_drive == TRUE;
    
    -- disable for signal driving --
    disable_drive : bool;
    keep soft disable_drive == FALSE;
    
    -- switch for interrupt feature --
    interrupt_enable : bool;
    keep soft interrupt_enable == TRUE;
    
    -- initial value --
    keep soft core_version       == "v3.x";
    keep soft driver_delay       == 1;       -- 2ns delay when driving terminals

    -- module category --
    module_type : module_category_t;
    keep soft module_type == NONE;
    
    post_generate() is also {
        outf("== LaijinBFM Configurations ======= \n");
        outf("  core_version       : %s \n",core_version           );
        outf("  id                 : %s \n",id                     );
        outf("  logger_on          : %s \n",logger_on              );
        outf("  driver_delay       : %d \n",driver_delay           );
        outf("  unknown_drive      : %s \n",unknown_drive          );
        outf("  disable_drive      : %s \n",disable_drive          );
        outf("  interrupt_enable   : %s \n",interrupt_enable       );
        outf("  pbus_start_address : 0x%06x \n",pbus_start_address );
        outf("=================================== \n");
    };
    
};

----------------------
-- type definitions --
----------------------
                       
type bus_state     : [ waiting_req, rdy_stall_one, rdy_stall_two, error ];
type stall_state   : [ no_stall, rdy_stall ];
type ready_state   : [ no_ready, ready ];
type request_state : [ no_request, request ];

type request_extend : [ no_extend, extend ];

------------------------------
-- method : cpu bus signals --
------------------------------
struct CPU_PBUS_SIGNALS {
    !req     : uint (bits:  1);
    !abus    : uint (bits: 24);
    !dbus    : uint (bits: 32);
    !ready   : uint (bits:  1);
    !dismiss : uint (bits:  1);
    
    set_init_values() is {
        req         = 1;
        abus        = 0;
        dbus        = 0;
        ready       = 1;
        dismiss     = 0;
    };
    
    post_generate() is also{
        set_init_values();
    };
    
};

struct CPU_BBUS_SIGNALS {
    !req     : uint (bits:  1);
    !abus    : uint (bits: 23);
    !dbus    : uint (bits: 32);
    !ready   : uint (bits:  1);
    
    set_init_values() is {
        req     = 1;
        abus    = 0;
        dbus    = 0;
        ready   = 1;
    };
    
    post_generate() is also{
        set_init_values();
    };
};

struct CPU_CBUS_SIGNALS {
    !req     : uint (bits:  1);
    !abus    : uint (bits: 23);
    !dbus    : uint (bits: 16);
    !ready   : uint (bits:  1);
    
    set_init_values() is {
        req     = 1;
        abus    = 0;
        dbus    = 0;
        ready   = 1;
    };
    
    post_generate() is also{
        set_init_values();
    };
};

struct CPU_DBUS_SIGNALS {
    !ioreq   : uint (bits:  1);
    !req     : uint (bits:  1);
    !abus    : uint (bits: 23);
    !dbus    : uint (bits: 16);
    !ready   : uint (bits:  1);
    
    ice_ready : uint (bits:  1);
    raw_ready : uint (bits:  1);
    
    set_init_values() is {
        ioreq   = 1;
        req     = 1;
        abus    = 0;
        dbus    = 0;
        ready   = 1;
        
        ice_ready = 1;
        raw_ready = 1;
    };
    
    post_generate() is also{
        set_init_values();
    };
};

struct CPU_EBUS_SIGNALS {
    !ioreq   : uint (bits:  1);
    !req     : uint (bits:  1);
    !abus    : uint (bits: 23);
    !dbus    : uint (bits: 16);
    !ready   : uint (bits:  1);
    
    ice_ready : uint (bits:  1);
    raw_ready : uint (bits:  1);
    
    set_init_values() is {
        ioreq   = 1;
        req     = 1;
        abus    = 0;
        dbus    = 0;
        ready   = 1;
        
        ice_ready = 1;
        raw_ready = 1;
    };
    
    post_generate() is also{
        set_init_values();
    };
};

struct CPU_FBUS_SIGNALS {
    !req     : uint (bits:  1);
    !abus    : uint (bits: 23);
    !dbus    : uint (bits: 16);
    !ready   : uint (bits:  1);
    
    set_init_values() is {
        req     = 1;
        abus    = 0;
        dbus    = 0;
        ready   = 1;
    };
    
    post_generate() is also{
        set_init_values();
    };
};

struct CPU_QUALIFIERS_SIGNALS {
    !wrbyte  : uint (bits:  1);
    !wrlsbe  : uint (bits:  1);
    !rdbyte  : uint (bits:  1);
    !rdlsbe  : uint (bits:  1);
    !doubler : uint (bits:  1);
    !doublew : uint (bits:  1);
    !dblcoef : uint (bits:  1);
    !lock    : uint (bits:  1);
    
    set_init_values() is {
        wrbyte  = 0;
        wrlsbe  = 0;
        rdbyte  = 0;
        rdlsbe  = 0;
        doubler = 0;
        doublew = 0;
        dblcoef = 0;
        lock    = 0;
    };
    
    post_generate() is also{
        set_init_values();
    };
};

struct CPU_EMULATION_SIGNALS {
    !dmapr       : uint (bits:  1);
    !dmapw       : uint (bits:  1);
    !dbugd       : uint (bits:  1);
    !dbuge       : uint (bits:  1);
    !debugenable : uint (bits:  1);
    !abortready  : uint (bits:  1);
    !suspend     : uint (bits:  1);
    
    set_init_values() is {
        dmapr       = 0;
        dmapw       = 0;
        dbugd       = 0;
        dbuge       = 0;
        debugenable = 0;
        abortready  = 0;
        suspend     = 1;
    };
    
    post_generate() is also{
        set_init_values();
    };
};

struct CPU_SYSTEM_SIGNALS {
    !progdismiss : uint (bits:  1);
    !gl_resetn   : uint (bits:  1);
    !cpu_resetn  : uint (bits:  1);
    !gl_mpnmc    : uint (bits:  1);
    !cpu_mpnmc   : uint (bits:  1);
    !gl_hompnsamp_init: uint (bits:  1);
    !gl_homrnsamr_init: uint (bits:  1);
    !gl_homxnsamx_init: uint (bits:  1);
    !gl_homynsamy_init: uint (bits:  1);
    !gl_homnsam  : uint (bits:  4);
    !cpu_homnsam : uint (bits:  4);
    !avis        : uint (bits:  1);
    !idlemaster  : uint (bits:  1);
    !idlewakeup  : uint (bits:  1);
    !idlecpu     : uint (bits:  1);
    !idlecpud    : uint (bits:  1);
    !int         : uint (bits: 22);
    !nmi         : uint (bits:  1);
    !iack        : uint (bits:  1);
    !buserror    : uint (bits:  1);
    !debugerror  : uint (bits:  1);
    !perrack     : uint (bits:  1);
    
    set_init_values() is {
        progdismiss = 1;
        gl_resetn   = 1;
        cpu_resetn  = 1;
        gl_mpnmc    = 0;
        cpu_mpnmc   = 0;
        gl_hompnsamp_init = 1;
        gl_homrnsamr_init = 1;
        gl_homxnsamx_init = 1;
        gl_homynsamy_init = 1;
        gl_homnsam  = 1;
        cpu_homnsam = 1;
        avis        = 0;
        idlemaster  = 0;
        idlewakeup  = 0;
        idlecpu     = 0;
        idlecpud    = 0;
        int         = 0x3fffff;
        nmi         = 1;
        iack        = 1;
        buserror    = 1;
        debugerror  = 1;
        perrack     = 0;
    };
    
    post_generate() is also {
        set_init_values();
    };
};

struct CPU_CACHE_SIGNALS {
    
    !cachefreeze : uint (bits:  1);
    !cacheenable : uint (bits:  1);
    !cacheclr    : uint (bits:  1);
    !cacheendclr : uint (bits:  1);
    
    set_init_values() is {
        cachefreeze = 0;
        cacheenable = 0;
        cacheclr    = 0;
        cacheendclr = 0;
    };
    
    post_generate() is also{
        set_init_values();
    };

};

struct CPU_MP_SIGNALS {

    -- PRIVILEGE / MEMORY PROTECTION
    !gl_privenable  : uint (bits:  1);
    !mpcxviol       : uint (bits:  1);
    !mpcbviol       : uint (bits:  1);
    !mpccviol       : uint (bits:  1);
    !mpcdviol       : uint (bits:  1);
    !mpceviol       : uint (bits:  1);
    !mpcfviol       : uint (bits:  1);
    !cpu_privenable : uint (bits:  1);
    !ppm            : uint (bits:  1);
    !bpm            : uint (bits:  1);
    !cpm            : uint (bits:  1);
    !dpm            : uint (bits:  1);
    !epm            : uint (bits:  1);
    !fpm            : uint (bits:  1);
    !rptpcvio       : uint (bits: 24);
    !rptintr        : uint (bits:  1);
    !rptpvio        : uint (bits:  1);
    !rptbvio        : uint (bits:  1);
    !rptcvio        : uint (bits:  1);
    !rptdvio        : uint (bits:  1);
    !rptiodvio      : uint (bits:  1);
    !rptevio        : uint (bits:  1);
    !rptioevio      : uint (bits:  1);
    !rptfvio        : uint (bits:  1);
    !rptpm          : uint (bits:  1);
    -- MemoryProtection for emulation
    !dbgmpviol      : uint (bits:  1);
    !xdmadmpovr     : uint (bits:  1);
    !xdmaempovr     : uint (bits:  1);
    !emiodready     : uint (bits:  1);
    !emioeready     : uint (bits:  1);
    
    set_init_values() is {
        gl_privenable  = 0;
        mpcxviol       = 0;
        mpcbviol       = 0;
        mpccviol       = 0;
        mpcdviol       = 0;
        mpceviol       = 0;
        mpcfviol       = 0;
        cpu_privenable = 0;
        ppm            = 0;
        bpm            = 0;
        cpm            = 0;
        dpm            = 0;
        epm            = 0;
        fpm            = 0;
        rptpcvio       = 0;
        rptintr        = 0;
        rptpvio        = 0;
        rptbvio        = 0;
        rptcvio        = 0;
        rptdvio        = 0;
        rptiodvio      = 0;
        rptevio        = 0;
        rptioevio      = 0;
        rptfvio        = 0;
        rptpm          = 0;
        dbgmpviol      = 0;
        xdmadmpovr     = 0;
        xdmaempovr     = 0;
        emiodready     = 1;
        emioeready     = 1;
    };
    
    post_generate() is also{
        set_init_values();
    };
    
};

unit CPU_SIGNALS {
    
    pbus       : CPU_PBUS_SIGNALS;
    bbus       : CPU_BBUS_SIGNALS;
    cbus       : CPU_CBUS_SIGNALS;
    dbus       : CPU_DBUS_SIGNALS;
    ebus       : CPU_EBUS_SIGNALS;
    fbus       : CPU_FBUS_SIGNALS;
    qualifiers : CPU_QUALIFIERS_SIGNALS;
    emulations : CPU_EMULATION_SIGNALS;
    systems    : CPU_SYSTEM_SIGNALS;
    cache      : CPU_CACHE_SIGNALS;
    mp         : CPU_MP_SIGNALS;
    
};

---------------------------
-- method : module paths --
---------------------------
unit CPU_SIGNAL_PATH {
    -- signal connention paths
    
    -- system clock --
    clock          : simple_port of uint is instance;
    
    -- system reset --
    gl_resetn      : simple_port of uint is instance;
    cpu_resetn     : simple_port of uint is instance;
    
    -- mpnmc related signals --
    gl_mpnmc       : simple_port of uint is instance;
    cpu_mpnmc      : simple_port of uint is instance;
    avis           : simple_port of uint is instance;

    -- HOM/SAM related signals --
    gl_hompnsamp_init : simple_port of uint is instance;
    gl_homrnsamr_init : simple_port of uint is instance;
    gl_homxnsamx_init : simple_port of uint is instance;
    gl_homynsamy_init : simple_port of uint is instance;
    gl_homnsam     : simple_port of uint is instance;
    cpu_homnsam    : simple_port of uint is instance;
    
    -- dismiss enable signals --
    progdismiss    : simple_port of uint is instance;
    
    -- idle --
    idlemaster     : simple_port of uint is instance;
    idlewakeup     : simple_port of uint is instance;
    idlecpu        : simple_port of uint is instance;
    idlecpud       : simple_port of uint is instance;
    
    -- interrupt --
    int            : simple_port of uint is instance;
    nmi            : simple_port of uint is instance;
    iack           : simple_port of uint is instance;
    
    -- buserror --
    buserror       : simple_port of uint is instance;
    debugerror     : simple_port of uint is instance;
    perrack        : simple_port of uint is instance;
    
    -- Pbus terminals(program read) --
    preq           : simple_port of uint is instance;
    pabus          : simple_port of uint is instance;
    pdbus          : simple_port of uint is instance;
    pready         : simple_port of uint is instance;
    pdismiss       : simple_port of uint is instance;
    -- Bbus terminals(coefficient read) --
    breq           : simple_port of uint is instance;
    babus          : simple_port of uint is instance;
    bdbus          : simple_port of uint is instance;
    bready         : simple_port of uint is instance;
    -- Cbus terminals(data read) --
    creq           : simple_port of uint is instance;
    cabus          : simple_port of uint is instance;
    cdbus          : simple_port of uint is instance;
    cready         : simple_port of uint is instance;
    -- Dbus terminals(data read) --
    iodreq         : simple_port of uint is instance;
    dreq           : simple_port of uint is instance;
    dabus          : simple_port of uint is instance;
    ddbus          : simple_port of uint is instance;
    dready         : simple_port of uint is instance;
    -- Ebus terminals(data write) --
    ioereq         : simple_port of uint is instance;
    ereq           : simple_port of uint is instance;
    eabus          : simple_port of uint is instance;
    edbus          : simple_port of uint is instance;
    eready         : simple_port of uint is instance;
    -- Fbus terminals(data write) --
    freq           : simple_port of uint is instance;
    fabus          : simple_port of uint is instance;
    fdbus          : simple_port of uint is instance;
    fready         : simple_port of uint is instance;
    -- R/W qualifier for byte operation --
    wrbyte         : simple_port of uint is instance;
    wrlsbe         : simple_port of uint is instance;
    rdbyte         : simple_port of uint is instance;
    rdlsbe         : simple_port of uint is instance;
    -- R/W qualifier for long read/write --
    doubler        : simple_port of uint is instance;
    doublew        : simple_port of uint is instance;
    dblcoef        : simple_port of uint is instance;
    -- R/W qualifier cpu lock signal --
    lock           : simple_port of uint is instance;
    -- Emulation qualifiers --
    dmapr          : simple_port of uint is instance;
    dmapw          : simple_port of uint is instance;
    dbugd          : simple_port of uint is instance;
    dbuge          : simple_port of uint is instance;
    debugenable    : simple_port of uint is instance;
    abortready     : simple_port of uint is instance;
    suspend        : simple_port of uint is instance;
    -- cache related signals --
    cachefreeze    : simple_port of uint is instance;
    cacheenable    : simple_port of uint is instance;
    cacheclr       : simple_port of uint is instance;
    cacheendclr    : simple_port of uint is instance;
    
    -- PRIVILEGE / MEMORY PROTECTION
    gl_privenable  : simple_port of uint is instance;
    mpcxviol       : simple_port of uint is instance;
    mpcbviol       : simple_port of uint is instance;
    mpccviol       : simple_port of uint is instance;
    mpcdviol       : simple_port of uint is instance;
    mpceviol       : simple_port of uint is instance;
    mpcfviol       : simple_port of uint is instance;
    cpu_privenable : simple_port of uint is instance;
    ppm            : simple_port of uint is instance;
    bpm            : simple_port of uint is instance;
    cpm            : simple_port of uint is instance;
    dpm            : simple_port of uint is instance;
    epm            : simple_port of uint is instance;
    fpm            : simple_port of uint is instance;
    rptpcvio       : simple_port of uint is instance;
    rptintr        : simple_port of uint is instance;
    rptpvio        : simple_port of uint is instance;
    rptbvio        : simple_port of uint is instance;
    rptcvio        : simple_port of uint is instance;
    rptdvio        : simple_port of uint is instance;
    rptiodvio      : simple_port of uint is instance;
    rptevio        : simple_port of uint is instance;
    rptioevio      : simple_port of uint is instance;
    rptfvio        : simple_port of uint is instance;
    rptpm          : simple_port of uint is instance;
    -- MemoryProtection for emulation
    dbgmpviol      : simple_port of uint is instance;
    xdmadmpovr     : simple_port of uint is instance;
    xdmaempovr     : simple_port of uint is instance;
    emiodready     : simple_port of uint is instance;
    emioeready     : simple_port of uint is instance;
    
        keep soft clock.hdl_path()             == "";
        keep soft gl_resetn.hdl_path()         == "";
        keep soft cpu_resetn.hdl_path()        == "";
        keep soft gl_mpnmc.hdl_path()          == "";
        keep soft cpu_mpnmc.hdl_path()         == "";
        keep soft gl_hompnsamp_init.hdl_path() == "";
        keep soft gl_homrnsamr_init.hdl_path() == "";
        keep soft gl_homxnsamx_init.hdl_path() == "";
        keep soft gl_homynsamy_init.hdl_path() == "";
        keep soft gl_homnsam.hdl_path()        == "";
        keep soft cpu_homnsam.hdl_path()       == "";
        keep soft avis.hdl_path()              == "";
        keep soft progdismiss.hdl_path()       == "";
        keep soft idlemaster.hdl_path()        == "";
        keep soft idlewakeup.hdl_path()        == "";
        keep soft idlecpu.hdl_path()           == "";
        keep soft idlecpud.hdl_path()          == "";
        keep soft int.hdl_path()               == "";
        keep soft nmi.hdl_path()               == "";
        keep soft iack.hdl_path()              == "";
        keep soft buserror.hdl_path()          == "";
        keep soft debugerror.hdl_path()        == "";
        keep soft perrack.hdl_path()           == "";
        keep soft preq.hdl_path()              == "";
        keep soft pabus.hdl_path()             == "";
        keep soft pdbus.hdl_path()             == "";
        keep soft pready.hdl_path()            == "";
        keep soft pdismiss.hdl_path()          == "";
        keep soft breq.hdl_path()              == "";
        keep soft babus.hdl_path()             == "";
        keep soft bdbus.hdl_path()             == "";
        keep soft bready.hdl_path()            == "";
        keep soft creq.hdl_path()              == "";
        keep soft cabus.hdl_path()             == "";
        keep soft cdbus.hdl_path()             == "";
        keep soft cready.hdl_path()            == "";
        keep soft iodreq.hdl_path()            == "";
        keep soft dreq.hdl_path()              == "";
        keep soft dabus.hdl_path()             == "";
        keep soft ddbus.hdl_path()             == "";
        keep soft dready.hdl_path()            == "";
        keep soft ioereq.hdl_path()            == "";
        keep soft ereq.hdl_path()              == "";
        keep soft eabus.hdl_path()             == "";
        keep soft edbus.hdl_path()             == "";
        keep soft eready.hdl_path()            == "";
        keep soft freq.hdl_path()              == "";
        keep soft fabus.hdl_path()             == "";
        keep soft fdbus.hdl_path()             == "";
        keep soft fready.hdl_path()            == "";
        keep soft wrbyte.hdl_path()            == "";
        keep soft wrlsbe.hdl_path()            == "";
        keep soft rdbyte.hdl_path()            == "";
        keep soft rdlsbe.hdl_path()            == "";
        keep soft doubler.hdl_path()           == "";
        keep soft doublew.hdl_path()           == "";
        keep soft lock.hdl_path()              == "";
        keep soft dmapr.hdl_path()             == "";
        keep soft dmapw.hdl_path()             == "";
        keep soft dbugd.hdl_path()             == "";
        keep soft dbuge.hdl_path()             == "";
        keep soft debugenable.hdl_path()       == "";
        keep soft abortready.hdl_path()        == "";
        keep soft suspend.hdl_path()           == "";
        keep soft cachefreeze.hdl_path()       == "";
        keep soft cacheenable.hdl_path()       == "";
        keep soft cacheclr.hdl_path()          == "";
        keep soft cacheendclr.hdl_path()       == "";

        keep soft gl_privenable.hdl_path()     == "";
        keep soft mpcxviol.hdl_path()          == "";
        keep soft mpcbviol.hdl_path()          == "";
        keep soft mpccviol.hdl_path()          == "";
        keep soft mpcdviol.hdl_path()          == "";
        keep soft mpceviol.hdl_path()          == "";
        keep soft mpcfviol.hdl_path()          == "";
        keep soft cpu_privenable.hdl_path()    == "";
        keep soft ppm.hdl_path()               == "";
        keep soft bpm.hdl_path()               == "";
        keep soft cpm.hdl_path()               == "";
        keep soft dpm.hdl_path()               == "";
        keep soft epm.hdl_path()               == "";
        keep soft fpm.hdl_path()               == "";
        keep soft rptpcvio.hdl_path()          == "";
        keep soft rptintr.hdl_path()           == "";
        keep soft rptpvio.hdl_path()           == "";
        keep soft rptbvio.hdl_path()           == "";
        keep soft rptcvio.hdl_path()           == "";
        keep soft rptdvio.hdl_path()           == "";
        keep soft rptiodvio.hdl_path()         == "";
        keep soft rptevio.hdl_path()           == "";
        keep soft rptioevio.hdl_path()         == "";
        keep soft rptfvio.hdl_path()           == "";
        keep soft rptpm.hdl_path()             == "";
        -- MemoryProtection for emulation
        keep soft dbgmpviol.hdl_path()         == "";
        keep soft xdmadmpovr.hdl_path()        == "";
        keep soft xdmaempovr.hdl_path()        == "";
        keep soft emiodready.hdl_path()        == "";
        keep soft emioeready.hdl_path()        == "";
    
    check_signal_connection( id : string ) is {
        if( clock.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : clock          \n",id) };
        if( gl_resetn.hdl_path()         == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : gl_resetn      \n",id) };
        if( cpu_resetn.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cpu_resetn     \n",id) };
        if( gl_mpnmc.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : gl_mpnmc       \n",id) };
        if( cpu_mpnmc.hdl_path()         == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cpu_mpnmc      \n",id) };
        if( gl_hompnsamp_init.hdl_path() == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : gl_hompnsamp_init     \n",id) };
        if( gl_homrnsamr_init.hdl_path() == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : gl_homrnsamr_init     \n",id) };
        if( gl_homxnsamx_init.hdl_path() == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : gl_homxnsamx_init     \n",id) };
        if( gl_homynsamy_init.hdl_path() == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : gl_homynsamy_init     \n",id) };
        if( gl_homnsam.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : gl_homnsam     \n",id) };
        if( cpu_homnsam.hdl_path()       == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cpu_homnsam    \n",id) };
        if( avis.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : avis           \n",id) };
        if( progdismiss.hdl_path()       == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : progdismiss    \n",id) };
        if( idlemaster.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : idlemaster     \n",id) };
        if( idlewakeup.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : idlewakeup     \n",id) };
        if( idlecpu.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : idlecpu        \n",id) };
        if( idlecpud.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : idlecpud       \n",id) };
        if( int.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : int            \n",id) };
        if( nmi.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : nmi            \n",id) };
        if( iack.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : iack           \n",id) };
        if( buserror.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : buserror       \n",id) };
        if( debugerror.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : debugerror     \n",id) };
        if( perrack.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : perrack        \n",id) };
        if( preq.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : preq           \n",id) };
        if( pabus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : pabus          \n",id) };
        if( pdbus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : pdbus          \n",id) };
        if( pready.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : pready         \n",id) };
        if( pdismiss.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : pdismiss       \n",id) };
        if( breq.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : breq           \n",id) };
        if( babus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : babus          \n",id) };
        if( bdbus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : bdbus          \n",id) };
        if( bready.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : bready         \n",id) };
        if( creq.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : creq           \n",id) };
        if( cabus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cabus          \n",id) };
        if( cdbus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cdbus          \n",id) };
        if( cready.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cready         \n",id) };
        if( iodreq.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : iodreq         \n",id) };
        if( dreq.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dreq           \n",id) };
        if( dabus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dabus          \n",id) };
        if( ddbus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : ddbus          \n",id) };
        if( dready.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dready         \n",id) };
        if( ioereq.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : ioereq         \n",id) };
        if( ereq.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : ereq           \n",id) };
        if( eabus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : eabus          \n",id) };
        if( edbus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : edbus          \n",id) };
        if( eready.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : eready         \n",id) };
        if( freq.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : freq           \n",id) };
        if( fabus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : fabus          \n",id) };
        if( fdbus.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : fdbus          \n",id) };
        if( fready.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : fready         \n",id) };
        if( wrbyte.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : wrbyte         \n",id) };
        if( wrlsbe.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : wrlsbe         \n",id) };
        if( rdbyte.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rdbyte         \n",id) };
        if( rdlsbe.hdl_path()            == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rdlsbe         \n",id) };
        if( doubler.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : doubler        \n",id) };
        if( doublew.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : doublew        \n",id) };
        if( lock.hdl_path()              == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : lock           \n",id) };
        if( dmapr.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dmapr          \n",id) };
        if( dmapw.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dmapw          \n",id) };
        if( dbugd.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dbugd          \n",id) };
        if( dbuge.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dbuge          \n",id) };
        if( debugenable.hdl_path()       == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : debugenable    \n",id) };
        if( abortready.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : abortready     \n",id) };
        if( suspend.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : suspend        \n",id) };
        if( cachefreeze.hdl_path()       == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cachefreeze    \n",id) };
        if( cacheenable.hdl_path()       == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cacheenable    \n",id) };
        if( cacheclr.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cacheclr       \n",id) };
        if( cacheendclr.hdl_path()       == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cacheendclr    \n",id) };
        if( gl_privenable.hdl_path()     == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : privenable     \n",id) };
        if( mpcxviol.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : mpcxviol       \n",id) };
        if( mpcbviol.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : mpcbviol       \n",id) };
        if( mpccviol.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : mpccviol       \n",id) };
        if( mpcdviol.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : mpcdviol       \n",id) };
        if( mpceviol.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : mpceviol       \n",id) };
        if( mpcfviol.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : mpcfviol       \n",id) };
        if( cpu_privenable.hdl_path()    == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cpu_privenable \n",id) };
        if( ppm.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : ppm            \n",id) };
        if( bpm.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : bpm            \n",id) };
        if( cpm.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : cpm            \n",id) };
        if( dpm.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dpm            \n",id) };
        if( epm.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : epm            \n",id) };
        if( fpm.hdl_path()               == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : fpm            \n",id) };
        if( rptpcvio.hdl_path()          == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptpcvio       \n",id) };
        if( rptintr.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptintr        \n",id) };
        if( rptpvio.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptpvio        \n",id) };
        if( rptbvio.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptbvio        \n",id) };
        if( rptcvio.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptcvio        \n",id) };
        if( rptdvio.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptdvio        \n",id) };
        if( rptiodvio.hdl_path()         == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptiodvio      \n",id) };
        if( rptevio.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptevio        \n",id) };
        if( rptioevio.hdl_path()         == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptioevio      \n",id) };
        if( rptfvio.hdl_path()           == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptfvio        \n",id) };
        if( rptpm.hdl_path()             == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : rptpm          \n",id) };
        if( dbgmpviol.hdl_path()         == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : dbgmpviol      \n",id) };
        if( xdmadmpovr.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : xdmadmpovr     \n",id) };
        if( xdmaempovr.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : xdmaempovr     \n",id) };
        if( emiodready.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : emiodready     \n",id) };
        if( emioeready.hdl_path()        == "" ) { outf("Warning of LaijinBFM(%s) -- unconnected path : emioeready     \n",id) };
    };
    
};

'>
