//********************************************************************************
//
//  Filename: cpubus_slave_structures.e
//
//  Description: extend of cpubus structs
//
//  Major Revision Descriptions:
//  oishi          09/11/02         new create in UMA2.0 branch
//
//*********************************************************************************
<'

-------------------------
-- added slave signals --
-------------------------
extend CPU_SIGNAL_PATH {
    
    -- Pbus terminals(program read) --
    slv_pdbus  : string;
    slv_pready : string;
    
    -- Bbus terminals(coefficient read) --
    slv_bdbus  : string;
    slv_bready : string;
    
    -- Cbus terminals(data read) --
    slv_cdbus  : string;
    slv_cready : string;
    
    -- Dbus terminals(data read) --
    slv_ddbus  : string;
    slv_dready : string;
    
    -- Ebus terminals(data write) --
    slv_eready : string;
    
    -- Fbus terminals(data write) --
    slv_fready : string;
    
    -- stall signals --
    slv_pstall : string;
    slv_rstall : string;
    slv_wstall : string;
    pstall     : string;
    rstall     : string;
    wstall     : string;
    
    init() is also {
        slv_pdbus  = "";
        slv_pready = "";
        slv_bdbus  = "";
        slv_bready = "";
        slv_cdbus  = "";
        slv_cready = "";
        slv_ddbus  = "";
        slv_dready = "";
        slv_eready = "";
        slv_fready = "";
        slv_pstall = "";
        slv_rstall = "";
        slv_wstall = "";
        pstall     = "";
        rstall     = "";
        wstall     = "";
    };
};

struct CPU_STALLS_SIGNALS {
    
    !slv_pstall : uint (bits: 1);
    !slv_rstall : uint (bits: 1);
    !slv_wstall : uint (bits: 1);
    
    !pstall     : uint (bits: 1);
    !rstall     : uint (bits: 1);
    !wstall     : uint (bits: 1);
    
    set_init_values() is {
        slv_pstall = 0;
        slv_rstall = 0;
        slv_wstall = 0;
        pstall     = 0;
        rstall     = 0;
        wstall     = 0;
    };
    
    post_generate() is also {
        set_init_values();
    };
};

struct CPU_STALLS {
    current  : CPU_STALLS_SIGNALS;
    previous : CPU_STALLS_SIGNALS;
    
    -- for detecting unknown/hiz --
    x        : CPU_STALLS_SIGNALS;
    z        : CPU_STALLS_SIGNALS;
    
    set_init_values() is {
        x.slv_pstall = 0; z.slv_pstall  = 0;
        x.slv_rstall = 0; z.slv_rstall  = 0;
        x.slv_wstall = 0; z.slv_wstall  = 0;
        x.pstall     = 0; z.pstall      = 0;
        x.rstall     = 0; z.rstall      = 0;
        x.wstall     = 0; z.wstall      = 0;
    };
    
    post_generate() is also{
        set_init_values();
    };
};
    
extend CPU_BUSES {
    
    stall : CPU_STALLS;
    
    keep_current_values() is also {
        
        -- keep signals --
        stall.previous = stall.current.copy();
    };
    
    get_current_value( config : CPUBUS_CONFIGURATIONS, paths : CPU_SIGNAL_PATH ) is also {
        var slv_pstall : uint (bits: 1);
        var slv_rstall : uint (bits: 1);
        var slv_wstall : uint (bits: 1);
        var pstall     : uint (bits: 1);
        var rstall     : uint (bits: 1);
        var wstall     : uint (bits: 1);
        
        -----------------------------
        -- for getting top signals --
        -----------------------------
        if( paths.slv_pstall != "" ) { slv_pstall = '(paths.slv_pstall)' } else { slv_pstall = 0 };
        if( paths.slv_rstall != "" ) { slv_rstall = '(paths.slv_rstall)' } else { slv_rstall = 0 };
        if( paths.slv_wstall != "" ) { slv_wstall = '(paths.slv_wstall)' } else { slv_wstall = 0 };
        if( paths.pstall     != "" ) { pstall     = '(paths.pstall)'     } else { pstall     = 0 };
        if( paths.rstall     != "" ) { rstall     = '(paths.rstall)'     } else { rstall     = 0 };
        if( paths.wstall     != "" ) { wstall     = '(paths.wstall)'     } else { wstall     = 0 };
        
        ------------------------
        -- set current values --
        ------------------------
        stall.current.slv_pstall = slv_pstall;
        stall.current.slv_rstall = slv_rstall;
        stall.current.slv_wstall = slv_wstall;
        stall.current.pstall     = pstall;
        stall.current.rstall     = rstall;
        stall.current.wstall     = wstall;
        
        ---------------------------
        -- get for unknown info. --
        ---------------------------
        if( paths.slv_pstall != "" ) { stall.x.slv_pstall = '(paths.slv_pstall)@x' } else { stall.x.slv_pstall = 0 };
        if( paths.slv_rstall != "" ) { stall.x.slv_rstall = '(paths.slv_rstall)@x' } else { stall.x.slv_rstall = 0 };
        if( paths.slv_wstall != "" ) { stall.x.slv_wstall = '(paths.slv_wstall)@x' } else { stall.x.slv_wstall = 0 };
        if( paths.pstall     != "" ) { stall.x.pstall     = '(paths.pstall)@x'     } else { stall.x.pstall     = 0 };
        if( paths.rstall     != "" ) { stall.x.rstall     = '(paths.rstall)@x'     } else { stall.x.rstall     = 0 };
        if( paths.wstall     != "" ) { stall.x.wstall     = '(paths.wstall)@x'     } else { stall.x.wstall     = 0 };
        
        -----------------------
        -- get for hiz info. --
        -----------------------
        if( paths.slv_pstall != "" ) { stall.x.slv_pstall = '(paths.slv_pstall)@z' } else { stall.x.slv_pstall = 0 };
        if( paths.slv_rstall != "" ) { stall.x.slv_rstall = '(paths.slv_rstall)@z' } else { stall.x.slv_rstall = 0 };
        if( paths.slv_wstall != "" ) { stall.x.slv_wstall = '(paths.slv_wstall)@z' } else { stall.x.slv_wstall = 0 };
        if( paths.pstall     != "" ) { stall.x.pstall     = '(paths.pstall)@z'     } else { stall.x.pstall     = 0 };
        if( paths.rstall     != "" ) { stall.x.rstall     = '(paths.rstall)@z'     } else { stall.x.rstall     = 0 };
        if( paths.wstall     != "" ) { stall.x.wstall     = '(paths.wstall)@z'     } else { stall.x.wstall     = 0 };
        
    };
};

--------------------------
-- update signal struct --
--------------------------
extend CPU_PBUS_SIGNALS {
    
    drive_slave_signals( paths : CPU_SIGNAL_PATH ) is {
        if( paths.slv_pdbus  != "" ) { '(paths.slv_pdbus)'  = dbus;  };
        if( paths.slv_pready != "" ) { '(paths.slv_pready)' = ready; };
    };
    
};

extend CPU_BBUS_SIGNALS {
    
    drive_slave_signals( paths : CPU_SIGNAL_PATH ) is {
        if( paths.slv_bdbus  != "" ) { '(paths.slv_bdbus)'  = dbus;  };
        if( paths.slv_bready != "" ) { '(paths.slv_bready)' = ready; };
    };
    
};

extend CPU_CBUS_SIGNALS {
    
    drive_slave_signals( paths : CPU_SIGNAL_PATH ) is {
        if( paths.slv_cdbus  != "" ) { '(paths.slv_cdbus)'  = dbus;  };
        if( paths.slv_cready != "" ) { '(paths.slv_cready)' = ready; };
    };
    
};

extend CPU_DBUS_SIGNALS {
    
    drive_slave_signals( paths : CPU_SIGNAL_PATH ) is {
        if( paths.slv_ddbus  != "" ) { '(paths.slv_ddbus)'  = dbus;  };
        if( paths.slv_dready != "" ) { '(paths.slv_dready)' = ready; };
    };
    
};

extend CPU_EBUS_SIGNALS {
    
    drive_slave_signals( paths : CPU_SIGNAL_PATH ) is {
        if( paths.slv_eready != "" ) { '(paths.slv_eready)' = ready; };
    };
    
};

extend CPU_FBUS_SIGNALS {
    
    drive_slave_signals( paths : CPU_SIGNAL_PATH ) is {
        if( paths.slv_fready != "" ) { '(paths.slv_fready)' = ready; };
    };
    
};

extend CPU_STALLS_SIGNALS {
    
    drive_slave_signals( paths : CPU_SIGNAL_PATH ) is {
        if( paths.slv_pstall != "" ) { '(paths.slv_pstall)'  = slv_pstall;  };
        if( paths.slv_rstall != "" ) { '(paths.slv_rstall)'  = slv_rstall;  };
        if( paths.slv_wstall != "" ) { '(paths.slv_wstall)'  = slv_wstall;  };
    };
    
};

extend CPU_SIGNALS {
    
    stall : CPU_STALLS_SIGNALS;

    drive_slave_signals( paths : CPU_SIGNAL_PATH, config : CPUBUS_CONFIGURATIONS  ) is {

        if( !config.disable_drive ) {
            pbus.drive_slave_signals( paths );
            bbus.drive_slave_signals( paths );
            cbus.drive_slave_signals( paths );
            dbus.drive_slave_signals( paths );
            ebus.drive_slave_signals( paths );
            fbus.drive_slave_signals( paths );
            stall.drive_slave_signals( paths );
        };
        
    };
    
};

'>
