//********************************************************************************
//
//  Filename: cpubus_watch_strcut.e
//
//  Description: for supporting watch struct.
//               This file is required only for using system stop_run management. 
//
//  Major Revision Descriptions:
//  oishi          09/08/02         copied from UMA1.0 branch
//  oishi          01/21/03         added simmode
//  mikep          01/22/03         completely remove any reference to simmode
//
//*********************************************************************************

--
-- when you use watch struct in your system,
-- 'STOP_RUN' and 'stop_run_management' should defined 
-- in testbench as following.
--
-- -- testbench.e --
-- define STOP_RUN sys.stop_run_management;
-- 
-- extend sys {
--     stop_run_management : watch_s;
-- };
--

---------------------------------------------------------
-- extend for supporting stop run management in system --
---------------------------------------------------------
<'
extend CPUBUS_MASTER_DRIVER {
 
  string_id : string;
  keep soft string_id == " laijin bfm ";
  
  watch_level : uint;
  keep soft watch_level == 0;
  
  watch_delay : uint;
  keep soft watch_delay == 10;
  
  id : uint;
  keep id == STOP_RUN.register_watch( string_id, watch_level, watch_delay );
  
  keep gen (watch_delay, watch_level, string_id) before (id);
  
  TCM_access_complete() @rise_clock is only {
      
      -- First, wait for higher priority bfm's to complete
      -- the zero parameter represent laijin's priority level
      STOP_RUN.wait_for_master( watch_level );
      
      -- Then wait for laijin to complete
      while( !(prog_access_complete and data_access_complete and
               data_queue.size() == 0 and program_queue.size() == 0 and
               bus_states.pbus.pending.counter == 0 and bus_states.bbus.pending.counter == 0 and
               bus_states.cbus.pending.counter == 0 and bus_states.dbus.pending.counter == 0 and
               bus_states.ebus.pending.counter == 0 and bus_states.fbus.pending.counter == 0     ) ) {
          wait cycle;
      };

      -- Now, report done, expecting for a watch_delay cycle delay
      STOP_RUN.report_done( watch_level, id );
      
  };
  
};

'>
